#include <cglib/gl/device_rendering_parameters.h>
#include <cglib/gl/device_rendering_context.h>
#include <cglib/core/assert.h>

#include <AntTweakBar.h>
#include <sstream>

#define LENGTH(a) ((sizeof(a))/sizeof((a)[0]))

static TwEnumVal renderer_enum[] = {
	{ DeviceRenderingParameters::SHADOW_MAP,           "Shadow Mapping" },
	{ DeviceRenderingParameters::FIRE_PLACE,           "FirePlace" },
	{ DeviceRenderingParameters::PROCEDURAL_LANDSCAPE, "Procedural Landscape" },
};

bool DeviceRenderingParameters::
derived_change_requires_restart(Parameters const& old_) const
{
	auto const* old = dynamic_cast<DeviceRenderingParameters const*>(&old_);
	cg_assert(old); // this would be a serious implementation error

#define CHECK_PARAM(a) || ((a) != old->a)
	const bool restart = false
		CHECK_PARAM(screen_width)
		CHECK_PARAM(screen_height)
		CHECK_PARAM(fov)
		CHECK_PARAM(near_clip)
		CHECK_PARAM(far_clip)
		CHECK_PARAM(current_renderer)
		CHECK_PARAM(enable_animation)

		CHECK_PARAM(shadow_map.shadow_bias)
		CHECK_PARAM(shadow_map.light_near_clip)
		CHECK_PARAM(shadow_map.light_far_clip)
		CHECK_PARAM(shadow_map.light_fov)
		CHECK_PARAM(shadow_map.light_direction)
		
		CHECK_PARAM(landscape.night_time)
		CHECK_PARAM(landscape.water.reflection_perturbation)
		CHECK_PARAM(landscape.water.wave_scaling)
		CHECK_PARAM(landscape.water.height)
		CHECK_PARAM(landscape.water.color)
		CHECK_PARAM(landscape.water.absorption)
		CHECK_PARAM(landscape.water.surface_absorption)
		CHECK_PARAM(landscape.water.enable_fresnel)
		CHECK_PARAM(landscape.terrain.height_scaling)
		CHECK_PARAM(landscape.terrain.beach_margin)
		CHECK_PARAM(landscape.terrain.min_rock_threshold)
		CHECK_PARAM(landscape.terrain.rock_blend_margin)
		CHECK_PARAM(landscape.terrain.lacunarity)
		CHECK_PARAM(landscape.terrain.weight_factor)
		CHECK_PARAM(landscape.terrain.angle_factor)
		CHECK_PARAM(landscape.terrain.valley_smoothness)
		CHECK_PARAM(landscape.visualize_heightmap)
		CHECK_PARAM(landscape.visualize_reflection)
		CHECK_PARAM(landscape.no_water_surface)
		CHECK_PARAM(landscape.no_textures)
		CHECK_PARAM(landscape.wireframe)
		CHECK_PARAM(landscape.quality_reduction)
		;
#undef CHECK_PARAM

	return restart;
}

static void TW_CALL
tw_reload_shaders(void *ClientData)
{
	(void) ClientData;
	DeviceRenderingContext::reload_all_shaders();
}

void DeviceRenderingParameters::
derived_gui_setup(TwBar *bar)
{
	cg_assert(bar);
	TwType render_mode_type = TwDefineEnum("Renderer", renderer_enum, LENGTH(renderer_enum));

	TwAddButton(bar, "btn_reload_shader", tw_reload_shaders, nullptr, "label='Reload all shaders'");

	TwAddVarRW(bar, "current_renderer", render_mode_type, &current_renderer, "label='Renderer' group='Rendering Settings'");

	if(0
	) {
		TwAddVarRW(bar, "animation", TW_TYPE_BOOLCPP, &enable_animation, "label='Animation' group='Rendering Settings'");
	}

	if(current_renderer == SHADOW_MAP) {
		TwAddVarRW(bar, "shadow_bias",     TW_TYPE_FLOAT, &shadow_map.shadow_bias,     "label='Shadow Bias'           group='Rendering Settings' min=0.0 step=0.0001");
		TwAddVarRW(bar, "light_near_clip", TW_TYPE_FLOAT, &shadow_map.light_near_clip, "label='Light near clip plane' group='Rendering Settings' min=0.0 step=0.01");
		TwAddVarRW(bar, "light_far_clip",  TW_TYPE_FLOAT, &shadow_map.light_far_clip,  "label='Light far clip plane'  group='Rendering Settings' min=0.0 step=0.01");
		TwAddVarRW(bar, "light_fov",       TW_TYPE_FLOAT, &shadow_map.light_fov,       "label='Light Fov'             group='Rendering Settings' min=10.0 max=140 step=1.0");
		TwAddVarRW(bar, "light_direction", TW_TYPE_DIR3F, &shadow_map.light_direction, "label='Light Direction'       group='Rendering Settings'");
	}
	if(current_renderer == PROCEDURAL_LANDSCAPE) {
		TwAddVarRW(bar, "visualize_heightmap", TW_TYPE_BOOLCPP, &landscape.visualize_heightmap, "label='Heightmap Visualization' group='Rendering Settings'");
		TwAddVarRW(bar, "visualize_reflection", TW_TYPE_BOOLCPP, &landscape.visualize_reflection, "label='Reflection Visualization' group='Rendering Settings'");
		TwAddVarRW(bar, "no_water_surface", TW_TYPE_BOOLCPP, &landscape.no_water_surface, "label='No Water Surface' group='Rendering Settings'");
		TwAddVarRW(bar, "no_textures", TW_TYPE_BOOLCPP, &landscape.no_textures, "label='No Textures' group='Rendering Settings'");
		TwAddVarRW(bar, "quality_reduction", TW_TYPE_INT32, &landscape.quality_reduction, "label='Reduce Quality' group='Rendering Settings' min=0 max=3");
		TwAddVarRW(bar, "wireframe", TW_TYPE_BOOLCPP, &landscape.wireframe, "label='Wireframe' group='Rendering Settings'");

		TwAddVarRW(bar, "night_time", TW_TYPE_BOOLCPP, &landscape.night_time, "label='Night Time' group='Scene Settings'");
		
		TwAddVarRW(bar, "enable_fresnel", TW_TYPE_BOOLCPP, &landscape.water.enable_fresnel, "label='Enable Fresnel' group='Water'");
		TwAddVarRW(bar, "water_height", TW_TYPE_FLOAT, &landscape.water.height, "label='Water Height' group='Water' step=0.1");
		TwAddVarRW(bar, "wave_scaling", TW_TYPE_FLOAT, &landscape.water.wave_scaling, "label='Wave Scaling' group='Water' min=0.0 step=0.01");
		TwAddVarRW(bar, "water_reflection_perturbation", TW_TYPE_FLOAT, &landscape.water.reflection_perturbation, "label='Reflection Perturbation' group='Water' min=0.0 max=1.0 step=0.01");
		TwAddVarRW(bar, "water_color", TW_TYPE_COLOR3F, &landscape.water.color, "label='Water Color' group='Water'");
		TwAddVarRW(bar, "water_absorption", TW_TYPE_FLOAT, &landscape.water.absorption, "label='Water Absorption' group='Water' min=0.0 step=0.01");
		TwAddVarRW(bar, "water_surface_absorption", TW_TYPE_FLOAT, &landscape.water.surface_absorption, "label='Water Surface Absorption' group='Water' min=0.0 step=0.01");
		TwDefine("Settings/Water group='Scene Settings'");

		TwAddVarRW(bar, "height_scaling", TW_TYPE_FLOAT, &landscape.terrain.height_scaling, "label='Height Scaling' group='Terrain' min=0.0 step=0.1");
		TwAddVarRW(bar, "beach_margin", TW_TYPE_FLOAT, &landscape.terrain.beach_margin, "label='Beach Margin' group='Terrain' step=0.01");
		TwAddVarRW(bar, "min_rock_threshold", TW_TYPE_FLOAT, &landscape.terrain.min_rock_threshold, "label='Min Rock Threshold' group='Terrain' min=0.0 step=0.01");
		TwAddVarRW(bar, "rock_blend_margin", TW_TYPE_FLOAT, &landscape.terrain.rock_blend_margin, "label='Rock Blend Margin' group='Terrain' min=0.0 max=1.0 step=0.01");

		TwAddVarRW(bar, "terrain_lacunarity", TW_TYPE_FLOAT, &landscape.terrain.lacunarity, "label='Lacunarity' group='Terrain' min=1.0 step=0.001");
		TwAddVarRW(bar, "terrain_weight_factor", TW_TYPE_FLOAT, &landscape.terrain.weight_factor, "label='Weight Factor' group='Terrain' min=0.0 step=0.001");
		TwAddVarRW(bar, "terrain_angle_factor", TW_TYPE_FLOAT, &landscape.terrain.angle_factor, "label='Angle Factor' group='Terrain' step=0.1");
		TwAddVarRW(bar, "terrain_valley_smoothness", TW_TYPE_FLOAT, &landscape.terrain.valley_smoothness, "label='Valley Smoothness' group='Terrain' min=0.0 max=1.0 step=0.01");
		TwDefine("Settings/Terrain group='Scene Settings'");
	}
}
